//+------------------------------------------------------------------+
//|                                                        iUSDx.mq5 |
//|                                            Copyright 2010, Lizar |
//|                            https://login.mql5.com/ru/users/Lizar |
//+------------------------------------------------------------------+
#define VERSION       "1.00 Build 1 (17 Dec 2010)"

#property copyright   "Copyright 2010, Lizar"
#property link        "https://login.mql5.com/ru/users/Lizar"
#property version     VERSION
#property description "iUSDx Multicurrency Indicator (USD Index)."
#property description "The indicator show the value of USD index via the currencies: EUR, JPY, GBP, CAD, SEK  CHF." 
#property description "The MCM Control Panel needed for the multicurrency mode"

//---- indicator properties:
#property indicator_separate_window        // indicator in separate window
#property indicator_plots 1                // 1 graphic plot used
#property indicator_buffers 7              // 8 buffers used
// first graphic plot - line:
#property indicator_type1  DRAW_LINE       // draw as a line
#property indicator_color1 Red             // line color
#property indicator_width1 2               // line width

#include <Control panel MCM.mqh> //<--- include MCM Control Panel
#include "iUSDx.mqh"

input int count=500; // Number of bars for the calculation

double iUSDx_Ind[];

string   currency="USD"; // USD
int      symbol_total=6; // Pairs, used in the calculation
int      calculate=0;

CUSDx *buffer[];
stCurrencyWeight currency_weight[];

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit()
  {   
   //--- MCM Control Panel initialization. 
   InitControlPanelMCM();

   ArrayResize(buffer,symbol_total);
   ArrayResize(currency_weight,symbol_total);
   
   currency_weight[0].symbol ="EURUSD"; currency_weight[0].weight =0.576;
   currency_weight[1].symbol ="USDJPY"; currency_weight[1].weight =0.136;
   currency_weight[2].symbol ="GBPUSD"; currency_weight[2].weight =0.119;
   currency_weight[3].symbol ="USDCAD"; currency_weight[3].weight =0.091;
   currency_weight[4].symbol ="USDSEK"; currency_weight[4].weight =0.042;
   currency_weight[5].symbol ="USDCHF"; currency_weight[5].weight =0.036;

   //--- indicator buffers mapping
   SetIndexBuffer(0,iUSDx_Ind,INDICATOR_DATA);          // iUSDx_Ind[] array is a buffer of the indicator

   for(int i=0; i<symbol_total; i++)
     {
      buffer[i]=new CUSDx;
      SetIndexBuffer(i+1,buffer[i].buffer,INDICATOR_CALCULATIONS); // buffer for intermediate calculations for ith symbol
      buffer[i].SetCurrencyAndWeight(currency_weight[i].symbol, currency_weight[i].weight);
     }

   ArraySetAsSeries(iUSDx_Ind,true);  // set indexation of iUSDx_Ind[] as time series

   //--- set accuracy
   IndicatorSetInteger(INDICATOR_DIGITS,_Digits);
   
   //--- name for DataWindow and indicator subwindow label
   IndicatorSetString(INDICATOR_SHORTNAME,"iUSDx "+VERSION);
   PlotIndexSetString(0,PLOT_LABEL,"iUSDx");
   return(0);
  }

//+------------------------------------------------------------------+
//| The standard event handler function.                             |
//| See MQL5 Reference for the details.                              |
//|                                                                  |
//| A convenient way to use the muticurreny mode,                    |
//| provided by MCM Control Panel.                                   |
//+------------------------------------------------------------------+
void OnChartEvent(const int id,         // event identifier or symbol position in "Market Watch"+CHARTEVENT_CUSTOM  
                  const long& lparam,   // event flag
                  const double& dparam, // price
                  const string& sparam  // symbol
                 )
  {
   int custom_id=id-CHARTEVENT_CUSTOM-1;
   
   if(custom_id>=0)      
     {
        if(lparam!=CHARTEVENT_NEWBAR_NO)
        {
         if(EventToPeriod(lparam)==_Period && sparam==_Symbol)
           { // recalculation of the indicator hen new bar had appeared on the current timeframe
            iUSDx_Ind[0]=EMPTY_VALUE;
            for(int i=0;i<symbol_total;i++) buffer[i].USDxSymbol();
            iUSDx(symbol_total);
            return;
           }
         
         buffer[custom_id].USDxSymbol();
         iUSDx(symbol_total);
         return;
        }
      else 
        { // recalculation of the indicator when "Initialization" event
         buffer[custom_id].InitSymbol(count,currency,sparam);
         Init_iUSDx(symbol_total,calculate);
         return;
        }
     }
  }
   
//+------------------------------------------------------------------+
//| Indicator buffer initialization                                  |
//| INPUT:  total       - number of symbols                          |
//|         count_iUSDx - number of bars for calculation             |
//| OUTPUT: no.                                                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
void Init_iUSDx(int total, int count_iUSDx)
  {   
   for(int bar=count_iUSDx-1; bar>=0; bar--) iUSDx(total,bar);     
   return;
  }

//+------------------------------------------------------------------+
//| Calculate the USD index for the specified bar                    |
//| INPUT:  total  - number of symbols                               |
//|         bar    - bar index                                       |
//| OUTPUT: no.                                                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
void iUSDx(int total, int bar=0)
  {
   iUSDx_Ind[bar]=50.14348112;
   for(int i=0;i<total;i++) 
     if(buffer[i].buffer[bar]!=EMPTY_VALUE && buffer[i].GetInit()) iUSDx_Ind[bar]*=buffer[i].buffer[bar];
     else { iUSDx_Ind[bar]=EMPTY_VALUE; return; }   
   if(iUSDx_Ind[bar]<5) iUSDx_Ind[bar]=EMPTY_VALUE;
   return;
  }

//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total, const int prev_calculated, const int begin, const double &price[]) 
  {
   if(prev_calculated==0)
     {
      ArrayInitialize(iUSDx_Ind,EMPTY_VALUE); 
      calculate=MathMin(rates_total,TerminalInfoInteger(TERMINAL_MAXBARS));   
      calculate=MathMin(calculate,count);
     }

   return(rates_total);
  }

//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
   for(int i=0;i<symbol_total;i++) delete buffer[i];   
   DeinitControlPanelMCM();   //<--- Deinitialize  MCM Control Panel:
  }
//+------------------------------------------------------------------+
